/* ============================= C MeatAxe ==================================
   File:        $Id: cfcomp.c,v 1.8 1998/09/16 15:39:26 mringe Exp $
   Comment:     
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <string.h>
#include <stdlib.h>


/* ------------------------------------------------------------------
   Global variables
   ------------------------------------------------------------------ */

static char *helptext[] = {
"SYNTAX",
"    cfcomp <Name> <Name2> ...",
"",
"OPTIONS",
"",
"FILES",
NULL};

static proginfo_t pinfo =
   { "cfcomp", "Compare irreducible constituents",
     "$Revision: 1.8 $", helptext };


/* ------------------------------------------------------------------
    GLobal data
   ------------------------------------------------------------------ */

Lat_Info InfoA;
matrix_t **CfGenA[LAT_MAXCF];
matrix_t **GenB;


/* ------------------------------------------------------------------
   init ()
   ------------------------------------------------------------------ */

static void Init(int argc, char *argv[])

{
    char fn[100];
    int i, k;

    MtxInit();
    MtxSetCommandLine(argc, argv, &pinfo);
    while ((i = MtxGetOpt("")) != OPT_END)
    {
    }
    if (argc < opt_ind + 2)
	MtxErrorExit("Bad number of arguments, try '%s -help'",pinfo.name);

    if (Lat_ReadInfo(&InfoA,argv[opt_ind]) != 0)
	MtxErrorExit("Cannot read %s.cfinfo",argv[opt_ind]);

    /* Read the generators for each composition factor
       ----------------------------------------------- */
    for (i = 0; i < InfoA.NCf; ++i)
    {
    	CfGenA[i] = NALLOC(matrix_t *,InfoA.NGen);
	for (k = 0; k < InfoA.NGen; ++k)
	{
	    sprintf(fn,"%s%s.%d",InfoA.BaseName,Lat_CfName(&InfoA,i),k+1);
	    MESSAGE(1,("Reading %s\n",fn));
	    CfGenA[i][k] = MatLoad(fn);
	    if (CfGenA[i][k] == NULL)
		MtxErrorExit("Cannot read %s",fn);
	}
    }
}



static void Cleanup()
{
    int i, k;
    for (i = 0; i < InfoA.NCf; ++i)
    {
 	for (k = 0; k < InfoA.NGen; ++k)
	    MatFree(CfGenA[i][k]);
    	FREE(CfGenA[i]);
    }
    MtxCleanup();
}



void ReadGens(const char *name)

{
    int k;
    char fn[300];

    GenB = NALLOC(matrix_t *,InfoA.NGen);
    for (k = 0; k < InfoA.NGen; ++k)
    {
	sprintf(fn,"%s.%d",name,k+1);
	MESSAGE(1,("Reading %s\n",fn));
	GenB[k] = MatLoad(fn);
    }
}


void FreeGens()

{
    int k;
    for (k = 0; k < InfoA.NGen; ++k)
	MatFree(GenB[k]);
    free(GenB);
}






/* ------------------------------------------------------------------
   FindEquiv()
   ------------------------------------------------------------------ */

static void FindEquiv(const char *name)

{
    int i;

    for (i = 0; i < InfoA.NCf; ++i)
    {
    	if (GenB[0]->nor != InfoA.Cf[i].dim) continue;
	if (IsIsomorphic(InfoA.NGen,(const matrix_t **)CfGenA[i],InfoA.Cf+i,
		(const matrix_t **)GenB, NULL,0))
	{
	    MESSAGE(0,("%s = %s%s\n",name,InfoA.BaseName,Lat_CfName(&InfoA,i)));
	    return;
	}
    }
    MESSAGE(0,("%s not found in %s\n",name,InfoA.BaseName));
}


/* ------------------------------------------------------------------
   Compare()
   ------------------------------------------------------------------ */

static void Compare(const char *name)
{
    ReadGens(name);
    FindEquiv(name);
    FreeGens(name);
}



/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, char *argv[])

{   
    int i;

    Init(argc, argv);
    for (i = opt_ind + 1; i < argc; ++i)
    {
	Compare(argv[i]);
    }
    Cleanup();
    return 0;
}


