/* ============================= C MeatAxe ==================================
   File:        $Id: init.c,v 1.2 1998/06/16 17:19:42 mringe Exp $
   Comment:     Initialization and clean up
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>
#include <stdlib.h>



int UseNewWordGenerator = 0;	/* HACK: global flag to select the word */
				/* generator version at run-time */


/*{{{ --------------------------------------------------------------------------
!function MtxInit "Initialize the library"
!section general.init
!synopsis 
    int MtxInit();
!return
    {\MeatAxe} version number, or -1 on error.
!description
    This function initializes the {\MeatAxe} library including finite field
    arithmetic and file i/o functions. It must be called before any other 
    {\MeatAxe} library function. \verb"MtxInit()" returns a version number 
    which is different for each implementation of the arithmetic. Currently 
    there are two versions, a small one for fields up to GF(256) and a big 
    version for field orders up to $2^{16}$.
!seealso MtxCleanup
-------------------------------------------------------------------------- }}}*/

int MtxInit()

{
    /* Initialize global variables
       --------------------------- */
    zchar = 0;
    zgen = 0;
    zfl = 0;		/* This forces zsetlen() to read the tables! */

    /* Perform platform-dependent initialization
       ----------------------------------------- */
    SInit();

    /* Return version number
       --------------------- */
    return (ZZZVERSION);
}


/*{{{ --------------------------------------------------------------------------
!function MtxCleanup "Terminate the library"
!section general.init
!synopsis 
    void MtxCleanup();
!description
    This function terminates the {\MeatAxe} library. Depending on the message
    level, some information like CPU time is written to stdout.
!seealso MtxInit
-------------------------------------------------------------------------- }}}*/

void MtxCleanup()

{
    long t = STimeUsed();
    MESSAGE(1,("Time used: %ld.%ld seconds\n",t/10,t%10));
}

