/* ============================= C MeatAxe ==================================
   File:        $Id: maddmul.c,v 1.7 1998/07/29 16:52:03 mringe Exp $
   Comment:     Add multiple of a matrix.
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"

/*{{{ --------------------------------------------------------------------------
!function MatAddMul "Add a multiple of a matrix"
!section mat.op
!synopsis 
    matrix_t *MatAddMul(matrix_t *dest, const matrix_t *src, FEL coeff);
!arg dest
    Matrix to add to.
!arg src
    Matrix to add.
!arg coeff
    Coefficient.
!return
    The function returns |dest|, or |NULL| on error.
!description
    This function adds a multiple of a matrix to another matrix. The 
    matrices must be compatible for addition.
    |MatAddMul()| handles special cases (|coeff| equals 0 or 1) in an
    intelligent way, so there is no need for the caller to do this.
!seealso MatAdd MatMul MatMulScalar
-------------------------------------------------------------------------- }}}*/

matrix_t *MatAddMul(matrix_t *dest, const matrix_t *src, FEL coeff)

{
    MTXCONTEXT(MatAddMul);

    /* Check the aguments
       ------------------ */
    if (!MATRIX_IS_VALID(src) || !MATRIX_IS_VALID(dest))
	return NULL;
    if (dest->fl != src->fl || dest->nor != src->nor || dest->noc != src->noc)
    {
	MTXERROR1("%E",ERR_INCOMPAT);
	return NULL;
    }

    /* Handle special cases
       -------------------- */
    if (coeff == F_ONE)
	MatAdd(dest,src);
    else if (coeff == F_ZERO)
	;
    else
    {
	/* Add multiple
	   ------------ */
	PTR dp = dest->d, sp = src->d;
	int n;
	zsetfield(src->fl);
	zsetlen(src->noc);
	for (n = src->nor; n > 0; --n)
	{
	    zaddmulrow(dp,sp,coeff);
	    zadvance(&dp,1);
	    zadvance(&sp,1);
	}
    }
    return dest;
}
