/* ============================= C MeatAxe ==================================
   File:        $Id: matadd.c,v 1.6 1998/07/29 16:52:04 mringe Exp $
   Comment:     Add two matrices
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"




/*{{{ --------------------------------------------------------------------------
!function MatAdd "Add a matrix"
!section mat.op
!synopsis 
    matrix_t *MatAdd(matrix_t *dest, const matrix_t *src);
!arg dest
    Matrix to add to.
!arg src
    Matrix to add.
!return
    The function returns |dest|, or |NULL| on error.
!description
    This function adds |src| to |dest|.  The matrices must be compatible,
    i.e., they must be over the same field and they muast have the same
    size.
!seealso MatAddMul MatMul MatMulScalar
-------------------------------------------------------------------------- }}}*/

matrix_t *MatAdd(matrix_t *dest, const matrix_t *src)

{
    MTXCONTEXT(MatAdd);
    PTR dp, sp;
    register int n;

    /* Check arguments
       --------------- */
    if (!MATRIX_IS_VALID(src) || !MATRIX_IS_VALID(dest))
	return NULL;
    if (dest->fl != src->fl || dest->nor != src->nor || dest->noc != src->noc)
    {
	MTXERROR1("%E",ERR_INCOMPAT);
	return NULL;
    }

    /* Add <src> to <dest>
       ------------------- */
    dp = dest->d;
    sp = src->d;
    zsetfield(src->fl);
    zsetlen(src->noc);
    for (n = src->nor; n > 0; --n)
    {
	zaddrow(dp,sp);
	zadvance(&dp,1);
	zadvance(&sp,1);
    }
    return dest;
}
