/* ============================= C MeatAxe ==================================
   File:        $Id: mattrace.c,v 1.3 1998/04/28 17:36:22 mringe Exp $
   Comment:     Trace of a matrix
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"



/*{{{ --------------------------------------------------------------------------
!function MatTrace "Trace of a matrix"
!section mat.op
!synopsis 
    FEL MatTrace(const matrix_t *mat);
!arg mat
    Pointer to the matrix.
!return
    The function returns the trace of |mat|. In case of error, the return 
    value is |F_ZERO|.
!description
    This function calculates the sum of all diagonal elements of a matrix.
    Note that the matrix need not be square.
!seealso
-------------------------------------------------------------------------- }}}*/

FEL MatTrace(const matrix_t *mat)

{
    MTXCONTEXT(MatTrace);
    int i;
    PTR x;
    int maxi;
    FEL trace = F_ZERO;

    /* Check the argument
       ------------------ */
    if (!MATRIX_IS_VALID(mat))
	return F_ZERO;

    maxi = mat->nor > mat->noc ? mat->noc : mat->nor;
    zsetfield(mat->fl);
    zsetlen(mat->noc);
    for (i = 1, x = mat->d; i <= maxi; ++i, zadvance(&x,1))
	trace = zadd(trace,zextract(x,i));
    return trace;
}
