/* ============================= C MeatAxe ==================================
   File:        $Id: mmulscal.c,v 1.3 1998/04/28 17:36:50 mringe Exp $
   Comment:     Multiply matrix by scalar.
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"



/*{{{ --------------------------------------------------------------------------
!function MatMulScalar "Calculate the multiple of a matrix"
!section mat.op
!synopsis 
    matrix_t *MatMulScalar(matrix_t *dest, FEL coeff);
!arg dest
    Pointer to the matrix.
!arg coeff
    Value to multiply with.
!return
    The function returns |dest|.
!description
    This function multiplies a matrix by a constant value.
!seealso MatAdd MatAddMul MatMul 
-------------------------------------------------------------------------- }}}*/

matrix_t *MatMulScalar(matrix_t *dest, FEL coeff)

{
    MTXCONTEXT(MatMulScalar);

    if (!MATRIX_IS_VALID(dest))
	return NULL;

    if (coeff == F_ONE)
    {
	/* Nothing to do */
    }
    else
    {
	PTR dp = dest->d;
	int n;
	zsetfield(dest->fl);
	zsetlen(dest->noc);
	for (n = dest->nor; n > 0; --n)
	{
	    zmulrow(dp,coeff);
	    zadvance(&dp,1);
	}
    }
    return dest;
}

