/* ============================= C MeatAxe ==================================
   File:        $Id: random.c,v 1.4 1998/06/16 17:19:44 mringe Exp $
   Comment:     Random number generator.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <limits.h>	/* For LONG_MAX */





/* ------------------------------------------------------------------
   Global data
   ------------------------------------------------------------------ */

static long int randtbl[32] =
{
    3,
    -851904987, -43806228, -2029755270, 1390239686, -1912102820,
    -485608943,1969813258, -1590463333,-1944053249,455935928,508023712,
    -1714531963, 1800685987, -2015299881, 654595283, -1149023258,
    -1470005550, -1143256056, -1325577603, -1568001885, 1275120390,
    -607508183, -205999574, -1696891592, 1492211999, -1528267240,
    -952028296, -189082757, 362343714, 1424981831, 2039449641,
};

static long int *fptr = &randtbl[4];
static long int *rptr = &randtbl[1];
static long int *state = &randtbl[1];
static long int *end_ptr = &randtbl[sizeof(randtbl)/sizeof(randtbl[0])];




/*{{{ --------------------------------------------------------------------------
!function MtxRandomInit "Initialize the random number generator"
!section kernel.misc
!synopsis 
    void MtxRandomInit(unsigned x);
!arg x
    Seed value.
!description
   This function initializes the random number generator.
!seealso @random
-------------------------------------------------------------------------- }}}*/

void MtxRandomInit(unsigned x)

{
    register int i;
    state[0] = x;
    for (i = 1; i < 31; ++i)
	state[i] = (1103515145 * state[i - 1]) + 12345;
    fptr = &state[3];
    rptr = &state[0];
    for (i = 0; i < 10 * 31; ++i)
	MtxRandom();
}



/*{{{ --------------------------------------------------------------------------
!function MtxRandom "Random number"
!section kernel.misc
!synopsis 
    long MtxRandom(void);
    #define MtxRandomInt(max) ((unsigned int) (MtxRandom() % (max)))
!return
    Pseudo random number.
!description
    The function |MtxRandom()| returns a (pseudo) random number. 
    |MtxRandomInt()| returns a (pseudo) random number in the range 0\ldots
    |max|.
!seealso @random
-------------------------------------------------------------------------- }}}*/

long MtxRandom(void)

{
      long int i;
      *fptr += *rptr;
      /* Chucking least random bit.  */
      i = (*fptr >> 1) & LONG_MAX;
      ++fptr;
      if (fptr >= end_ptr)
	{
	  fptr = state;
	  ++rptr;
	}
      else
	{
	  ++rptr;
	  if (rptr >= end_ptr)
	    rptr = state;
	}
      return i;
}




