/* ============================= C MeatAxe ==================================
   File:        $Id: zad.c,v 1.3 1998/04/28 17:37:51 mringe Exp $
   Comment:     Add two matrices.
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include <string.h>
#include "meataxe.h"



/* ------------------------------------------------------------------
   Variables
   ------------------------------------------------------------------ */

static char *helptext[] = {
"SYNTAX",
"    zad <Mat1> <Mat2> <Result>",
"",
"FILES",
"    <Mat1>    i  First matrix",
"    <Mat2>    i  Second matrix. Must be of the same type as <Mat1>.",
"    <Result>  o  The result",
NULL};

static proginfo_t pinfo =
   { "zad", "Add Matrices", "$Revision: 1.3 $", helptext };




/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, char *argv[])

{
    long nor, noc, nor2, noc2, fl, fl2;
    FILE *file1, *file2, *file3;
    PTR m1, m2;
    register long i;

    MtxInit();

    /* Parse command line
       ------------------ */
    MtxSetCommandLine(argc, argv, &pinfo);
    while (MtxGetOpt("") != OPT_END);
    if ((i = opt_ind) != argc-3) 
	MtxErrorExit("%s: %E\n",pinfo.name,ERR_BADUSAGE);

    /* Open input files
       ---------------- */
    if ((file1 = zreadhdr(argv[opt_ind],&fl,&nor,&noc)) == NULL)
	MtxErrorExit("Cannot open %s: %E",argv[opt_ind],MtxErrNo);
    if (fl < 2) 
	MtxErrorExit("%s is not a matrix",argv[opt_ind]);
    if ((file2 = zreadhdr(argv[opt_ind+1],&fl2,&nor2,&noc2)) == NULL)
	MtxErrorExit("Cannot open %s: %E",argv[opt_ind+1],MtxErrNo);
    if (fl2 < 2) 
	MtxErrorExit("%s is not a matrix",argv[opt_ind+1]);
    if (fl != fl2 || nor != nor2 || noc != noc2)
	MtxErrorExit("%s and %s are not compatible",argv[opt_ind],argv[opt_ind+1]);
    zsetfield(fl);
    zsetlen(noc);
    file3 = zwritehdr(argv[opt_ind+2],fl,nor,noc);
    if (file3 == NULL)
	MtxErrorExit("Cannot write %s: %E",argv[opt_ind+2],MtxErrNo);

    /* Add matrices row by row
       ----------------------- */
    m1 = zalloc(1);
    m2 = zalloc(1);
    for (i = nor; i > 0; --i)
    {
	if (zreadvec(file1,m1,1) != 1) 
	    MtxErrorExit("Cannot read %s: %E",argv[opt_ind],MtxErrNo);
	if (zreadvec(file2,m2,1) != 1) 
	    MtxErrorExit("Cannot read %s: %E",argv[opt_ind+1],MtxErrNo);
	zaddrow(m1,m2);
	if (zwritevec(file3,m1,1) != 1)
	    MtxErrorExit("Cannot write %s: %E",argv[opt_ind+2],MtxErrNo);
    }
    fclose(file1);
    fclose(file2);
    fclose(file3);
    MtxCleanup();
    return EXIT_OK;
}

