/* ============================= C MeatAxe ==================================
   File:        $Id: zcl.c,v 1.3 1998/04/28 17:37:54 mringe Exp $
   Comment:     Clean matrix.
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <string.h>
#include <stdlib.h>




/* ------------------------------------------------------------------
   Variables
   ------------------------------------------------------------------ */

static char *helptext[] = {
"SYNTAX",
"    zcl <Subsp> <Mat> <Cleaned mat> <Ops>",
"",
"FILES",
"    <Subsp>    i  The subspace to clean with",
"    <Mat>      i  The matrix to be cleaned.",
"    <Cleaned>  o  The cleaned matrix",
"    <Ops>      o  Contains the operations that were performed, i.e.",
"                  <Mat> = <Ops> * <Subsp> + <Cleaned>",
NULL};

static proginfo_t pinfo =
   { "zcl", "Clean Matrix", "$Revision: 1.3 $", helptext };


static char *matname, *subname, *clname, *opname;



int main(int argc, char *argv[])

{
    PTR mat, row, op;
    long fl, fl2;
    long nor1, noc, nor2, noc2;
    long j;
    long *piv;
    int i;
    FILE *f, *fcl, *fop;


    /* Parse command line
       ------------------ */
    MtxSetCommandLine(argc, argv, &pinfo);
    while ((i = MtxGetOpt("")) != OPT_END)
    {
    }

    if (opt_ind != argc-4) 
	MtxErrorExit("%s: %E",pinfo.name,ERR_NARGS);
    subname = argv[opt_ind];
    matname = argv[opt_ind+1];
    clname = argv[opt_ind+2];
    opname = argv[opt_ind+3];

    /* Open files and initialize
       ------------------------- */
    if ((f = zreadhdr(subname,&fl,&nor1,&noc)) == NULL)
	MtxErrorExit(NULL);
    if (fl <= 1) 
	MtxErrorExit("%s: %E",subname,ERR_NOTMATRIX);
    piv = NALLOC(long,nor1+1);
    if (piv == NULL) 
	MtxErrorExit(NULL);
    zsetfield(fl);
    zsetlen(noc);
    mat = zalloc(nor1);
    row = zalloc((long)1);
    zreadvec(f,mat,nor1);
    fclose(f);
    if (zmkpivot(mat,nor1,piv) == -1) /* Make pivot table */
	MtxErrorExit(NULL);
    if ((f = zreadhdr(matname,&fl2,&nor2,&noc2)) == NULL)
	MtxErrorExit(NULL);
    if (noc2 != noc || fl2 != fl)
	MtxErrorExit("%s and %s: %E",subname,matname,ERR_INCOMPAT);
    zsetlen(nor1);
    op = zalloc((long)1);
    zmulrow(op,F_ZERO);
    if ((fcl = zwritehdr(clname,fl,nor2,noc)) == NULL)
	MtxErrorExit(NULL);
    if ((fop = zwritehdr(opname,fl,nor2,nor1)) == NULL)
	MtxErrorExit(NULL);

    /* Process matrix row by row
       ------------------------- */
    for (j = 1; j <= nor2; ++j)
    {
	zsetlen(noc);
	zreadvec(f,row,1);		/* Read next row */
	zcleanrow2(row,mat,nor1,piv,op);/* Clean with matrix */
	zwritevec(fcl,row,1);		/* Write cleaned row */
	zsetlen(nor1);
	zwritevec(fop,op,1);		/* Write operation */
	zmulrow(op,F_ZERO);
    }
    fclose(f);
    fclose(fop);
    fclose(fcl);
    MtxCleanup();
    return (EXIT_OK);
}

