/* ============================= C MeatAxe ==================================
   File:        $Id: zef.c,v 1.3 1998/04/28 17:38:03 mringe Exp $
   Comment:     Reduce to (normalized) echelon form
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */



#include "meataxe.h"
#include <stdlib.h>




/* ------------------------------------------------------------------
   Global data
   ------------------------------------------------------------------ */

static long *piv;
static int opt_G = 0;	/* GAP output */
static char *iname, *oname;
static FILE *ifile, *ofile;

static char *helptext[] = {
"SYNTAX",
"    zef [-GQV] <Inp> <Out>",
"",
"OPTIONS",
"    -G   GAP Output format",
"    -Q   Quiet, no messages",
"    -V   Verbose, more messages",
"",
"FILES",
"    <Inp>    i  A matrix",
"    <Out>    o  Matrix reduced to echelon form",
NULL};

static proginfo_t pinfo =
   { "zef", "Echelon form", "$Revision: 1.3 $", helptext };



/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, char *argv[])

{
    int i;
    long k, rank, fl, nor, noc;
    PTR m1, x;


    /* Parse command line
       ------------------ */
    MtxInit();
    MtxSetCommandLine(argc, argv, &pinfo);
    while ((i = MtxGetOpt("G")) != OPT_END)
    {
	switch (i)
	{
	    case 'G': opt_G = 1; msg_level = -100; break;
	}
    }

    if (opt_ind != argc-2)
	MtxErrorExit("%s: %E\n",pinfo.name,ERR_BADUSAGE);
    iname = argv[opt_ind];
    oname = argv[opt_ind+1];
	
    if ((ifile = zreadhdr(iname,&fl,&nor,&noc)) == NULL)
	MtxErrorExit(NULL);
    if (fl < 1)
	MtxErrorExit("%s: %E",iname,ERR_NOTMATRIX);
    if (MSG1) printf("%s: %ldx%ld-Matrix over GF(%ld)\n",
	iname,nor,noc,fl);
    piv = NALLOC(long,nor+1);
    if (piv == NULL)
	MtxErrorExit(NULL);
    zsetfield(fl);
    zsetlen(noc);
    m1 = zalloc(nor);
    zreadvec(ifile,m1,nor);
    fclose(ifile);

    /* Reduce to echelon form and normalize
       ------------------------------------ */
    rank = zmkechelon(m1,nor,piv);
    for (x=m1,k=1; k <= rank; ++k, zadvance(&x,(long)1))
	zmulrow(x,zinv(zextract(x,piv[k])));

    /* Write result
       ------------ */
    if ((ofile = zwritehdr(oname,fl,rank,noc)) == NULL)
	MtxErrorExit(NULL);
    zwritevec(ofile,m1,rank);
    fclose(ofile);
    if (opt_G)
	printf("MeatAxe.Rank := %ld;\n",rank);
    else
        MESSAGE(0,("RANK %ld\n",rank));
    MtxCleanup();
    return (EXIT_OK);
}

