/* ============================= C MeatAxe ==================================
   File:        $Id: zfr.c,v 1.4 1998/04/30 19:19:02 mringe Exp $
   Comment:     Apply the Frobenius automorphism to a matrix.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"


/* ------------------------------------------------------------------
   Global data
   ------------------------------------------------------------------ */

static char *helptext[] = {
"SYNTAX",
"    zfr [-QV] <Matrix> <Result>",
"",
"OPTIONS",
"    -Q   Quiet, no messages",
"    -V   Verbose, more messages",
"",
"FILES",
"    <Matrix>    i  A matrix",
"    <Result>    o  The result, a matrix of the same size",
"",
NULL};
static proginfo_t pinfo =
   { "zfr", "Frobenius Map", "$Revision: 1.4 $", helptext };

char *iname, *oname;
FILE *ifile, *ofile;


/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, char *argv[])

{
    long fl;			/* Field */
    PTR m1;				/* One row of the matrix */
    long nor, noc;			/* Number of rows and columns */
    long i, k, n, p;
    FEL f1, f2;

    /* Parse command line
       ------------------ */
    MtxInit();
    MtxSetCommandLine(argc, argv, &pinfo);
    while (MtxGetOpt("") != OPT_END);
    if (argc - opt_ind != 2) 
	MtxErrorExit("%s: %E",pinfo.name,ERR_NARGS);
    iname = argv[opt_ind];
    oname = argv[opt_ind+1];

    /* Open the input file
       ------------------- */
    if ((ifile = zreadhdr(iname,&fl,&nor,&noc)) == NULL)
	MtxErrorExit(NULL);
    if (fl < 2) 
	MtxErrorExit("%s: %E",iname,ERR_NOTMATRIX);
    zsetfield(fl); 
    zsetlen(noc);
    p = zchar;
    MESSAGE(0,("CHARACTERISTIC IS %ld\n",p));

    /* Open output file, allocate memory
       --------------------------------- */
    if ((ofile = zwritehdr(oname,fl,nor,noc)) == NULL)
	MtxErrorExit(NULL);
    m1 = zalloc(1);

    /* Apply the frobenius map to each entry
       ------------------------------------- */
    for (i = 1; i <= nor; ++i)
    {
	zreadvec(ifile,m1,(long) 1);
	for (k = 1; k <= noc; ++k)
	{
	    f1 = zextract(m1,k);
	    f2 = f1;
	    for (n = p - 1; n != 0; --n)
		f2 = zmul(f1,f2);
	    zinsert(m1,k,f2);
	}
	zwritevec(ofile,m1,(long) 1);
    }

    fclose(ifile);
    fclose(ofile);
    MtxCleanup();
    return (EXIT_OK);
}


