/* ============================= C MeatAxe ==================================
   File:        $Id: zgap.c,v 1.3 1998/03/25 17:42:25 mringe Exp $
   Comment:     Convert between MeatAxe and GAP format.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"



/*{{{ --------------------------------------------------------------------------
!function zftogap "Convert to GAP format"
!section kernel.io
!synopsis 
    char *zftogap(FEL f);
!arg f
    Field element.
!return
    Pointer to the GAP representtion of |f|.
!description
    This function takes a field element and returns the GAP
    representation of this element. The return value is a pointer
    to a static buffer which is overwritten on each call.
-------------------------------------------------------------------------- }}}*/

char *zftogap(FEL f)

{
    static char buffer[40];

    if (zchar == zfl)	/* Prime field */
    {
	FEL f2 = F_ZERO;
   	long k = 0;
    	while (f2 != f)
    	{   f2 = zadd(f2,zgen);
	    ++k;
	}
	sprintf(buffer,"%ld*Z(%ld)",k,zfl);
    }
    else		/* Other field */
    {
	if (f == F_ZERO)
	    sprintf(buffer,"0*Z(%ld)",zfl);
	else
	{
	    FEL f2 = zgen;
	    long k = 1;
	    while (f2 != f)
	    {   
		f2 = zmul(f2,zgen);
		++k;
	    }
	    sprintf(buffer,"Z(%ld)^%ld",zfl,k);
	}
    }
    return buffer;
}

