/* ============================= C MeatAxe ==================================
   File:        $Id: ziv.c,v 1.4 1998/04/28 17:38:15 mringe Exp $
   Comment:     Invert a matrix or permutation.
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */



#include <stdlib.h>
#include "meataxe.h"



/* ------------------------------------------------------------------
   Global data
   ------------------------------------------------------------------ */

static char *iname, *oname;
static FILE *ifile, *ofile;
static long fl;
static long nor, noc;

static char *helptext[] = {
"SYNTAX",
"    ziv <File> <Result>",
"",
"FILES",
"    <File>    i  A square matrix or permutation",
"    <Result>  o  The result",
NULL};

static proginfo_t pinfo =
   { "ziv", "Invert Matrix Or Permutation",
     "$Revision: 1.4 $", helptext };



/* ------------------------------------------------------------------
   invperm() - Invert a permutation.
   ------------------------------------------------------------------ */

void invperm()

{
    long *p1 = NULL, *p2 = NULL;
    long i;

    if (noc > 1)
    {   fprintf(stderr,
	     "ZIV WARNING - ONLY 1 OF %ld ELEMENTS INVERTED\n",noc);
    }
    if ((p1 = NALLOC(long,nor)) == NULL ||
	(p2 = NALLOC(long,nor)) == NULL)
	MtxErrorExit("Cannot allocate work space");
    if (zreadlong(ifile,p1,nor) != (size_t) nor)
	MtxErrorExit("Cannot read %s",iname);

    /* Calculate inverse in p2
       ----------------------- */
    --p1;
    --p2;
    for (i = 1; i <= nor; ++i)
	p2[p1[i]] = i;
    if ((ofile = zwritehdr(oname,fl,nor,(long)1)) == NULL)
	MtxErrorExit(NULL);
    if (zwritelong(ofile,p2+1,nor) != (size_t) nor)
	MtxErrorExit("Cannot write to %s",oname);
    fclose(ofile);
}


/* ------------------------------------------------------------------
   invmat() - Invert a matrix.
   ------------------------------------------------------------------ */

void invmat()

{
    PTR mat, result;

    /* Read matrix
       ----------- */
    zsetfield(fl); 
    zsetlen(noc);
    if (nor != noc) 
	MtxErrorExit("%s: Matrix not square",iname);
    mat = zalloc(nor);
    result = zalloc(nor);
    if (zreadvec(ifile,mat,nor) != (size_t) nor) 
	MtxErrorExit("Cannot read from %s: %S",iname);

    /* Make inverse
       ------------ */
    if (zmatinv(mat,result)) 
	MtxErrorExit("Cannot invert %s: %E",iname,MtxErrNo);

    /* Write result
       ------------ */
    if ((ofile = zwritehdr(oname,fl,nor,noc)) == NULL)
	MtxErrorExit("Cannot open %s for output: %S",oname);
    zwritevec(ofile,result,nor);
    fclose(ofile);
}



int main(int argc, char *argv[])

{

    /* Parse command line
       ------------------ */
    MtxInit();
    MtxSetCommandLine(argc, argv, &pinfo);
    while (MtxGetOpt("") != OPT_END)
    {
    }
    if (opt_ind != argc-2) 
	MtxErrorExit("%s: %E\n",pinfo.name,ERR_BADUSAGE);
    iname = argv[opt_ind];
    oname = argv[opt_ind+1];
    if ((ifile = zreadhdr(iname,&fl,&nor,&noc)) == NULL)
	MtxErrorExit("Cannot open %s: %S",iname);
    if (fl < 0)
       	invperm();
    else
       	invmat();
    fclose(ifile);
    MtxCleanup();
    return (EXIT_OK);
}



