/* ============================= C MeatAxe ==================================
   File:        $Id: znu.c,v 1.3 1998/04/28 17:38:23 mringe Exp $
   Comment:     This program calculates the null space of a matrix.
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */



#include <stdlib.h>
#include "meataxe.h"





/* ------------------------------------------------------------------
   Variables
   ------------------------------------------------------------------ */

static char *helptext[] = {
"SYNTAX",
"    znu [-GQV] <Matrix> [<NullSpace>]",
"",
"OPTIONS",
"    -G     GAP output",
"    -Q     Quiet, no messages",
"    -V     Verbose, more messages",
"",
"FILES",
"    <Matrix>       i  Any matrix.",
"    <NullSpace>    o  The null space.",
NULL};

static proginfo_t pinfo =
   { "znu", "Matrix Null-Space", "$Revision: 1.3 $", helptext };


int opt_G = 0;
static char *matname;
static char *nspname = NULL;
static FILE *matfile, *nspfile;



/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, char *argv[])

{   long nor, noc, dim;
    long *piv;
    long fl;
    int i;
    PTR m1, m2;

    /* Parse command line
       ------------------ */
    MtxInit();
    MtxSetCommandLine(argc, argv, &pinfo);
    while ((i = MtxGetOpt("G")) != OPT_END)
    {
	switch (i)
	{
	    case 'G': opt_G = 1; msg_level = -100; break;
	}
    }
    switch (argc - opt_ind)
    {
        case 2:
	    nspname = argv[opt_ind+1];
        case 1:
	    matname = argv[opt_ind];
	    break;
	default:
	    MtxErrorExit("%s: %E\n",pinfo.name,ERR_BADUSAGE);
    }

    /* Read input file, check sizes
       ---------------------------- */
    if ((matfile = zreadhdr(matname,&fl,&nor,&noc)) == NULL)
	MtxErrorExit("Cannot open %s: %S",matname);
    if (fl < 0) 
	MtxErrorExit("%s: %E",matname,ERR_NOTMATRIX);
    piv = NALLOC(long,nor+1);
    zsetfield(fl);
    zsetlen(nor);
    m2 = zalloc(nor);
    zsetlen(noc);
    m1 = zalloc(nor);
    zreadvec(matfile,m1,nor);
    fclose(matfile);

    /* Calculate null-space
       -------------------- */
    dim = znullsp(m1,nor,piv,m2);
    if (opt_G)
    {
	printf("MeatAxe.Nullity := %ld;\n",dim);
	fflush(stdout);
    }
    else
        MESSAGE(0,("NULLITY %ld\n",dim));

    /* Write out the null space
       ------------------------ */
    if (nspname != NULL)
    {
        if (MSG1) printf("Writing null-space to %s\n",nspname);
    	if ((nspfile = zwritehdr(nspname,fl,dim,nor)) == NULL)
	    MtxErrorExit("Cannot open %s for output: %S",nspname);
    	zsetlen(nor);
	zwritevec(nspfile,m2,dim);
        fclose(nspfile);
    }
    MtxCleanup();
    return 0;
}



