/* ============================= C MeatAxe ==================================
   File:        $Id: zqt.c,v 1.3 1998/04/28 17:38:30 mringe Exp $
   Comment:     Projection on quotient.
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include <string.h>
#include <stdlib.h>
#include "meataxe.h"




/* ------------------------------------------------------------------
   Global data
   ------------------------------------------------------------------ */

static int opt_i = 0;
static PTR subsp, mat, tmp, quot;
static long fl, nor, noc, sdim, qdim;
static char *mname, *sname, *oname;
static FILE *mfile, *sfile, *ofile;


static char *helptext[] = {
"SYNTAX",
"    zqt [-i] [<Subsp> <Matrix> <Quotient>]",
"",
"OPTIONS",
"    -i   Take only insignificant rows of <Matrix>. <Quotient> will be",
"         the action of <Matrix> on the quotient by <Subspace>.",
"",
"FILES",
"    <Subsp>    i  The invariant subspace, in semi-echelon form",
"    <Matrix>   i  The matrix, must have the same number of columns",
"    <Quotient> o  Insignificant columns of <Matrix>, cleaned with <Subspace>",
"",
NULL};

static proginfo_t pinfo =
   { "zqt", "Clean And Quotient", "$Revision: 1.3 $", helptext };


/* ------------------------------------------------------------------
   err() - Print error message and exit
   ------------------------------------------------------------------ */

static void err(c)
int c;

{
    fprintf(stderr,"ZQT ERROR - ");
    switch (c)
    {
	case 'm':
	    fprintf(stderr,"MATRICES NOT COMPATIBLE\n");
	    break;
	case 'e':
	    fprintf(stderr,"SUBSPACE NOT IN ECHELON FORM\n");
	    break;
	default:
	    fprintf(stderr,"UNKNOWN ERROR `%c'\n",c);
	    break;
    }
    exit(1);
}


/* ------------------------------------------------------------------
   init() - Process command line options and arguments
   ------------------------------------------------------------------ */

static void init(int argc, char *argv[])

{   int i;

    /* Parse command line
       ------------------ */
    MtxInit();
    MtxSetCommandLine(argc, argv, &pinfo);
    while ((i = MtxGetOpt("i")) != OPT_END)
    {
	switch (i)
	{
	    case 'i': opt_i = 1; break;
	}
    }
    if (opt_ind != argc-3) 
	MtxErrorExit("%s: %E",ERR_BADUSAGE);
    sname = argv[opt_ind];
    mname = argv[opt_ind+1];
    oname = argv[opt_ind+2];
}


/* ------------------------------------------------------------------
   readfiles() - Read the input files, allocate tables
   ------------------------------------------------------------------ */

void readfiles()

{
    long fls, nors, nocs, flm, norm, nocm;

    if ((sfile = zreadhdr(sname,&fls,&nors,&nocs)) == NULL)
	MtxErrorExit(NULL);
    if ((mfile = zreadhdr(mname,&flm,&norm,&nocm)) == NULL)
	MtxErrorExit(NULL);
    if (nocs != nocm || fls != flm) err('m');
    if (opt_i && norm != nocm) 
	MtxErrorExit("%s: %E",mname,ERR_NOTSQUARE);
    zsetfield(fls);
    zsetlen(nocs);
    subsp = zalloc(nors);
    zreadvec(sfile,subsp,nors);
    mat = zalloc(norm);
    tmp = zalloc((long)1);
    zreadvec(mfile,mat,norm);
    fclose(sfile);
    fclose(mfile);
    fl = flm;
    noc = nocm;
    nor = norm;
    sdim = nors;
    qdim = noc - nors;
}


/* ------------------------------------------------------------------
   doit()
   ------------------------------------------------------------------ */

void doit()

{
    long *piv = NALLOC(long,sdim+1);
    zsetfield(fl); 
    zsetlen(noc);
    if (zmkpivot(subsp,sdim,piv) != 0) 
	err('e');
    zquotinit(subsp,sdim,piv);
    if (opt_i)
    {
        zsetlen(qdim);
	quot = zalloc(qdim);
	zquotop(mat,quot);
    }
    else
    {
	zsetlen(sdim);
	quot = zalloc(nor);
	zquot(mat,nor,quot);
    }
}


/* ------------------------------------------------------------------
   writefile()
   ------------------------------------------------------------------ */

void writefile()

{
    long resnor = opt_i ? qdim : nor;
	
    zsetlen(qdim);
    if ((ofile = zwritehdr(oname,fl,resnor,qdim)) == NULL)
    	MtxErrorExit(NULL);
    zwritevec(ofile,quot,resnor);
    fclose(ofile);
}


/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, char *argv[])

{
    init(argc, argv);
    readfiles();
    doit();
    writefile();
    MtxCleanup();
    return 0;
}

