/* ============================= C MeatAxe ==================================
   File:        $Id: ztc.c,v 1.3 1998/04/28 17:38:44 mringe Exp $
   Comment:     Trace of a matrix or permutation.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <stdlib.h>



/* ------------------------------------------------------------------
   Global data
   ------------------------------------------------------------------ */

long fl, nor, noc;
int opt_G = 0;		/* GAP output */
int msg = 0;
FILE *inp;
char *inpname;

static char *helptext[] = {
"SYNTAX",
"    ztc [-GQV] <File>",
"",
"OPTIONS",
"    -G   GAP output",
"    -V   Verbose output (repeat to increase number of messages)",
"    -Q   Quiet, no messages",
"",
"FILES",
"    <File>    i  A square matrix or permutation(s)",
"",
NULL};

static proginfo_t pinfo =
   { "ztc", "Trace", "$Revision: 1.3 $", helptext };




/* ------------------------------------------------------------------
   trmat() - Trace of a matrix
   ------------------------------------------------------------------ */

static void trmat()

{
    FEL tr;
    long i, max;
    PTR m1;

    zsetfield(fl); zsetlen(noc);
    m1 = zalloc((long)1);
    tr = F_ZERO;
    if ((max = nor) > noc) max = noc;
    for (i = 1; i <= max; ++i)
    {
	zreadvec(inp,m1,1);
	tr = zadd(tr,zextract(m1,i));
    }
    if (!opt_G)		/* Standard output */
	printf("TRACE IS %ld\n",zftoi(tr));
    else		/* GAP output */
        printf("MeatAxe.Trace := %s;\n",zftogap(tr));
}


/* ------------------------------------------------------------------
   trperm() - Trace of a permutation (= number of fixed points)
   ------------------------------------------------------------------ */

static void trperm()

{
    long *m1;
    long i,k,tr=0;

    m1 = NALLOC(long,nor);
    if (m1 == NULL) 
	MtxErrorExit(NULL);


    if (opt_G)
	printf("MeatAxe.Trace := [");
    for (i = 1; i <= noc; ++i)
    {
	if (zreadlong(inp,m1,nor) != (size_t) nor)
	    MtxErrorExit(NULL);
	tr = 0;
	--m1;
	for (k = 1; k <= nor; ++k)
	    if (m1[k] == k) ++tr;

	if (!opt_G)	/* Standard output */
	    printf("ELEMENT %3ld HAS TRACE %6ld\n",i,tr);
	else		/* GAP output */
	{
	    if (i > 1) printf(",");
	    if (i % 10 == 0) printf("\n   ");
	    printf("%ld",tr);
	}
    }
    if (opt_G)
	printf("];\n");
}




/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, char *argv[])


{
    int i;

    /* Parse command line
       ------------------ */
    MtxInit();
    MtxSetCommandLine(argc, argv, &pinfo);
    while ((i = MtxGetOpt("G")) != OPT_END)
    {
	switch (i)
	{
	    case 'G': opt_G = 1; msg_level = -100; break;
	}
    }
    if (opt_ind != argc-1) 
	MtxErrorExit("%s: %E",pinfo.name,ERR_USAGE);
    inpname = argv[opt_ind];
    if ((inp = zreadhdr(inpname,&fl,&nor,&noc)) == NULL)
	MtxErrorExit(NULL);
    if (msg >= 1)
    {
	printf("Input is ");
	if (fl == -1)
	    printf("%ld permutation(s) of degree %ld\n",noc,nor);
	else
	    printf("a %ldx%ld matrix over GF(%ld)\n",nor,noc,fl);
    }
    if (fl < -1) 
	MtxErrorExit("%s: bad type (%d)",inpname,(int)fl);
    if (fl < 0)
	trperm();
    else
	trmat();
    fclose(inp);
    MtxCleanup();
    return (EXIT_OK);
}




