/* ============================= C MeatAxe ==================================
   File:        $Id: ztr.c,v 1.4 1998/04/28 17:38:48 mringe Exp $
   Comment:     Transpose a matrix.
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"



/* ------------------------------------------------------------------
   Global Data
   ------------------------------------------------------------------ */

static char *iname, *oname;

static char *helptext[] = {
"SYNTAX",
"    ztr [-QV] <Mat> <Result>",
"",
"OPTIONS",
"    -Q    Quiet, no messages",
"    -V    Verbatim, more messages",
"",
"FILES",
"    <Mat>      i  Any matrix",
"    <Result>   o  The transposed matrix",
NULL};

static proginfo_t pinfo =
   { "ztr", "Transpose", "$Revision: 1.4 $", helptext };



/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, char *argv[])

{
    long fl;
    PTR m1, m2;
    long nor, noc, j;
    FILE *f;


    /* Parse command line
       ------------------ */
    MtxInit();
    MtxSetCommandLine(argc, argv, &pinfo);
    while (MtxGetOpt("") != OPT_END)
    {
    }
    if (opt_ind != argc-2) 
	MtxErrorExit("%s: %E\n",pinfo.name,ERR_BADUSAGE);
    iname = argv[opt_ind];
    oname = argv[opt_ind+1];

    /* Read matrix
       ----------- */
    if ((f = zreadhdr(iname,&fl,&nor,&noc)) == NULL)
	MtxErrorExit("Cannot open %s: %S",iname);
    if (fl < 2) 
	MtxErrorExit("%s: %E",iname,ERR_NOTMATRIX);
    zsetfield(fl);
    zsetlen(noc);
    m1 = zalloc(nor);
    zreadvec(f,m1,nor);
    fclose(f);

    /* Transpose
       --------- */
    zsetlen(nor);
    m2 = zalloc((long)1);
    if ((f = zwritehdr(oname,fl,noc,nor)) == NULL)
	MtxErrorExit("Cannot open %s for output: %S",oname);
    for (j = 1; j <= noc; ++j)
    {
	zsetlen(nor);
	zmulrow(m2,F_ZERO);	/* Fill with zeros */
	zsetlen(noc);
	zextractcol(m1,nor,j,m2);
	zsetlen(nor);
	zwritevec(f,m2,(long)1);
    }
    fclose(f);
    MtxCleanup();
    return (EXIT_OK);
}


