/* ============================= C MeatAxe ==================================
   File:        $Id: zuk.c,v 1.3 1998/04/28 17:38:49 mringe Exp $
   Comment:     Uncondense
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <stdlib.h>
#include <string.h>





/* ------------------------------------------------------------------
   Global data
   ------------------------------------------------------------------ */

static char *helptext[] ={
"SYNTAX",
"    zuk <Vectors> <Orbits> <Result>",
"",
"FILES",
"    <Vectors>    i  A matrix (#columns = #orbits)",
"    <Orbits>     i  An orbit sizes file",
"    <Result>     o  Uncondensed vectors, a matrix",
NULL};

static proginfo_t pinfo =
   { "zuk", "Uncondense Vectors", "$Revision: 1.3 $", helptext };

static char *vecname, *orbname, *resname;


/* ------------------------------------------------------------------
   err() - Exit with error message
   ------------------------------------------------------------------ */

static void err(ch)
int ch;

{
    fprintf(stderr,"ZUK ERROR - ");
    switch (ch)
    {
	case '=':
	    fprintf(stderr,"MATRIX AND ORBIT SIZES ARE INCOMPATIBLE\n");
	    break;
    }
    exit(EXIT_ERR);
}




/* ------------------------------------------------------------------
   uncondense()
   ------------------------------------------------------------------ */

static void uncondense()

{
    long fl, nor, noc, fl2, norb, noc2, noc3;
    long *perm;
    PTR inp, out;
    long *os, *x;
    long i;
    FILE *vecfile, *orbfile, *resfile;

    /* Open the vector file
       -------------------- */
    if ((vecfile = zreadhdr(vecname,&fl,&nor,&noc)) == NULL)
	MtxErrorExit(NULL);
    if (fl < 2) 
	MtxErrorExit("%s: %E",vecname,ERR_NOTMATRIX);
    zsetfield(fl);
    zsetlen(noc);
    inp = zalloc((long)1);

    /* Read the orbit sizes
       -------------------- */
    if ((orbfile = zreadhdr(orbname,&fl2,&noc2,&norb)) == NULL)
	MtxErrorExit(NULL);
    if (fl2 != -T_IMAT || noc2 != 1)
	MtxErrorExit("%s: bad file format",orbname);
    if (norb != noc) 
	err('=');
    perm = NALLOC(long,norb);
    if (perm == NULL) 
	MtxErrorExit(NULL);
    if (zreadlong(orbfile,perm,norb) != (size_t) norb)
	MtxErrorExit(NULL);
    fclose(orbfile);
    os = (long *)perm - 1;

    /* Calculate length of output vectors
       ---------------------------------- */
    x = os + 1;
    noc3 = 0;
    for (i = 1; i <= norb; ++i)
    {
	noc3 += *x;
	++x;
    }
    MESSAGE(0,("OUTPUT IS %ld x %ld\n",nor,noc3));
    zsetlen(noc3);
    out = zalloc((long)1);
    if ((resfile = zwritehdr(resname,fl,nor,noc3)) == NULL)
	MtxErrorExit(NULL);

    /* Uncondense row by row
       --------------------- */
    for (i = 1; i <= nor; ++i)
    {
	long k, outpos = 1;

	zsetlen(noc);
	zreadvec(vecfile,inp,1);
	zsetlen(noc3);
	zmulrow(out,F_ZERO);
	x = os + 1;
	for (k = 1; k <= norb; ++k)
	{
	    FEL f;
	    long l;

	    zsetlen(noc);
	    f = zextract(inp,k);
	    zsetlen(noc3);
	    for (l = 1; l <= *x; ++l)
		zinsert(out,outpos++,f);  
	    ++x;
	}
	zsetlen(noc3);
	zwritevec(resfile,out,1);
    }
    fclose(resfile);
    fclose(vecfile);
}




/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, char *argv[])

{
    MtxInit();

    /* Parse command line
       ------------------ */
    MtxSetCommandLine(argc, argv, &pinfo);
    while (MtxGetOpt("") != OPT_END)
    {
    }
    if (opt_ind != argc-3) 
	MtxErrorExit("%s: %E\n",pinfo.name,ERR_BADUSAGE);
    vecname = argv[opt_ind];
    orbname = argv[opt_ind+1];
    resname = argv[opt_ind+2];
    uncondense();
    MtxCleanup();
    return (EXIT_OK);
}


