/* ============================= C MeatAxe ==================================
   File:        $Id: zzztest.c,v 1.6 1998/05/28 17:33:58 mringe Exp $
   Comment:     Test the arithmetic module.
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */



#include "meataxe.h"
#include <stdlib.h>
#include <string.h>


#if defined (BIG)
#define ISFEL(f) ((f) == 0xffff ||\
	(unsigned short)(f) < (unsigned short)fld-1)
#else
#define ISFEL(f) ((unsigned int)(f) < (unsigned int)fld)
#endif

#define TNOC 20	  /* Number of colums for testing row operations */



/* ------------------------------------------------------------------
   Global variables
   ------------------------------------------------------------------ */

int zzzvers;
long fld;
FEL *ftab;
int opt_t = 0;
int opt_b = 0;

char amsg[] = "zzztest aborted";

static char *helptext[] = {
"SYNTAX",
"    zzztest [-t] [-b] <Field> ...",
"",
"OPTIONS",
"    -t     Print tables",
"    -b     Benchmark",
NULL};

static proginfo_t pinfo =
   { "zzztest", "Test finite field functions", "$Revision: 1.6 $",
      helptext };











#define ERR(msg) err(__LINE__,msg)

void err(int line,char *msg)

{	printf("Line %d: %s\n",line,msg);
	fflush(stdout);
	exit(1);
}




void testitof()

{
    long l, m;

    if (!opt_b && !opt_t)
	printf("Testing int <--> FEL conversion\n");
    if (zitof((long)0) != F_ZERO) ERR("Error 1");
    if (zitof((long)1) != F_ONE) ERR("Error 2");
    if (zftoi(F_ZERO) != 0) ERR("Error 3");
    if (zftoi(F_ONE) != 1) ERR("Error 4");
    for (l = 0; l < fld; ++l)
    {	ftab[l] = zitof(l);
	if (!ISFEL(ftab[l])) ERR("Error 5");
	for (m = 0; m < l; ++m)
	    if (ftab[m] == ftab[l]) ERR("Error 6");
	if (zftoi(ftab[l]) != l) ERR("Error 7");
    }
}



void testfield()

{   FEL a,b,c;
    int ai, bi, ci;
	
    printf("Testing field arithmetic\n");

    /* Test one and zero elements
       -------------------------- */
    for (ai = 0; ai < (int)fld; ++ai)
    {	a = ftab[ai];
	if (zadd(a,F_ZERO) != a)
	{   printf("%d+0=%d\n",(int)a,(int)zadd(a,F_ZERO));
	    ERR(amsg);
	}
	if (zmul(a,F_ONE) != a)
	{	printf("%d*1=%d\n",(int)a,(int)zmul(a,F_ONE));
		ERR(amsg);
	}
    }

    /* Test negative and inverse
       ------------------------- */
    for (ai = 0; ai < (int)fld; ++ai)
    {	a = ftab[ai];
	b = zneg(a);
	if (!ISFEL(b) || zadd(a,b) != F_ZERO) 
	    ERR("Illegal negative");
	if (a != F_ZERO)
	{   b = zinv(a);
	    if (!ISFEL(b) || zmul(a,b) != F_ONE) 
		ERR("Illegal inverse");
	}
    }

    /* Test arithmetic
       --------------- */
    for (ai = 0; ai < (int)fld; ++ai)
    {	a = ftab[ai];
	for (bi = ai; bi < (int)fld; ++bi)
	{   b = ftab[bi];
	    c = zadd(a,b);
	    if (!ISFEL(c)) ERR("zadd() error");
	    if (c != zadd(b,a)) ERR("'+' not commutative");
	    c = zmul(a,b);
	    if (!ISFEL(c)) ERR("zmul() error");
	    if (c != zmul(b,a)) ERR("'*' not commutative");
	    
	    for (ci = 0; ci < (int)fld; ++ci)
	    {	c = ftab[ci];
		if (zadd(a,zadd(b,c)) != zadd(zadd(a,b),c))
		    ERR("'+' not associative");
		if (zmul(a,zmul(b,c)) != zmul(zmul(a,b),c))
		    ERR("'*' not associative");
		if (zmul(a,zadd(b,c)) != zadd(zmul(a,b),zmul(a,c)))
		    ERR("a*(b+c) != a*b+a*c");
	    }
	}
    }
}


void testgen()
{
    FEL a, b;
    int i;

    MESSAGE(0,("Testing zgen\n"));
    a = zgen;
    MESSAGE(1,("Generator = %d\n",(int) zgen));
    b = a;
    for (i = 1; i < (int)fld-1; ++i)
    {	
	if (b == F_ONE)
	{
	    ERR("Generator test failed");
	    break;
	}
	b = zmul(a,b);
    }
    if (b != F_ONE)
    	ERR("Generator test failed");
}


void testextractinsert()

{
    long noc;

    printf("Testing zinsert()/zextract()\n");
    for (noc = 1; noc < 35; ++noc)
    {
	PTR x;
	unsigned i;

	zsetfield(fld);
	zsetlen(noc);
	x = zalloc((long)1);
  	for (i = 0; i < 50; ++i)
	{
	    long col;
	    MtxRandomInit(i);
	    for (col = 1; col <= noc; ++col)
		zinsert(x,col,ftab[MtxRandomInt(fld)]);
	    MtxRandomInit(i);
	    for (col = 1; col <= noc; ++col)
		if (zextract(x,col) != ftab[MtxRandomInt(fld)])
		    ERR("insert/extract test failed");
	}
	free(x);
    }
}


void testfindmark()

{	
    PTR x;
    FEL a;
    long i;

    zsetfield(fld);
    zsetlen(TNOC);
    printf("Testing zfindpiv()\n");
    x = zalloc((long)1);

    for (i = 1; i <= TNOC; ++i)
	zinsert(x,i,ftab[i%(fld-1)+1]);
    for (i = 1; i <= TNOC; ++i)
    {	
	if (zfindpiv(x,&a) != i) ERR("Error 1");
	if (a != ftab[i%(fld-1)+1]) ERR("Error 2");
	zinsert(x,i,F_ZERO);
    }
    free(x);
}


void testaddrow()

{	PTR x, y;
	int a, b;
	FEL c;
	long i;

	zsetfield(fld);
	zsetlen(TNOC);
	printf("Testing zaddrow()\n");
	x = zalloc((long)1);
	y = zalloc((long)1);
	for (a = 0; a < (int)fld; ++a)
	    for (b = 0; b < (int)fld; ++b)
	    {	for (i = 1; i <= TNOC; ++i)
		{	zinsert(x,i,ftab[a]);
			zinsert(y,i,ftab[b]);
		}
		zaddrow(x,y);
		c = zadd(ftab[a],ftab[b]);
                for (i = 1; i <= TNOC; ++i)
		{	if (zextract(x,i) != c)
				ERR("Test failed");
		}
	    }
	free(x);
	free(y);
}



void testrowops()

{	PTR m1,m2,m3;
	long i, k;
	int d;

	zsetfield(fld);
	zsetlen(TNOC);
	printf("Testing row operations\n");
	m1 = zalloc((long)1);
	m2 = zalloc((long)1);
	m3 = zalloc((long)1);

	/* Test zmoverow() and zcmprow()
	   ----------------------------- */
	for (i = 1; i <= TNOC; ++i)
		zinsert(m1,i,ftab[i%fld]);
	for (i = 1; i <= TNOC; ++i)
	{	zmoverow(m2,m1);
		if (zcmprow(m2,m1)) ERR("Error 1");
		if (memcmp(m2,m1,zsize((long)1))) ERR("Error 2");
		zinsert(m1,i,ftab[(i+1)%fld]);
		if (!zcmprow(m2,m1)) ERR("Error 3");
	}

	/* Test zaddrow() and zaddmulrow()
	   ------------------------------- */
	for (d = 0; d < 6; ++d)
	{	for (i = 1; i <= TNOC; ++i)
		{	zinsert(m1,i,ftab[i%fld]);
			zinsert(m2,i,ftab[(i+d)%fld]);
		}
		zmoverow(m3,m1);
		zaddrow(m3,m2);
		for (i = 1; i <= TNOC; ++i)
		{	if (zextract(m3,i) !=
			    zadd(ftab[i%fld],ftab[(i+d)%fld]))
				ERR("Error 4");
		}
		for (k = 0; k < fld; ++k)
		{	zmoverow(m3,m1);
			zaddmulrow(m3,m2,ftab[k]);
			for (i = 1; i <= TNOC; ++i)
			{	if (zextract(m3,i) != zadd(ftab[i%fld],
				    zmul(ftab[(i+d)%fld],ftab[k])))
					ERR("Error 5");
			}
		}
	}
}



static poly_t *RandomPol(int deg)

{
    poly_t *a = PolAlloc(fld,deg);
    int i;
    for (i = 0; i < (int)a->deg; ++i)
	a->buf[i] = zitof(MtxRandomInt(fld));
    a->buf[i] = zitof(MtxRandomInt(fld-1)+1);
    return a;
}

static matrix_t *RandomMat(int nor, int noc)

{
    matrix_t *a = MatAlloc(fld,nor,noc);
    int i;
    PTR x;

    for (i = 1, x = a->d; i <= nor; ++i, zadvance(&x,1))
    {
	int k;
	for (k = 1; k <= noc; ++k)
	    zinsert(x,k,zitof(MtxRandomInt(fld)));
    }
    return a;
}

static perm_t *RandomPerm(int deg)

{
    perm_t *a = PermAlloc(deg);
    int i;

    for (i = 0; i < deg; ++i)
    {
	int x = MtxRandomInt(deg);
	int y = MtxRandomInt(deg);
	long tmp = a->d[x];
	a->d[x] = a->d[y];
	a->d[y] = tmp;
    }
    return a;
}

int MatCompare(const matrix_t *mat1, const matrix_t *mat2)

{
    if (mat1->fl != mat2->fl)
	return 1;
    if (mat1->nor != mat2->nor)
	return 1;
    if (mat1->noc != mat2->noc)
	return 1;
    zsetlen(mat1->noc);
    return memcmp(mat1->d,mat2->d,zsize(mat1->nor));
}


int PermCompare(const perm_t *perm1, const perm_t *perm2)

{
    if ( perm1->deg != perm1->deg )
	return 1;
    return memcmp(perm1->d,perm2->d,perm1->deg * sizeof(long));
}

void TestReadWrite()

{
    poly_t *pol1, *pol2;
    matrix_t *mat1, *mat2;
    perm_t *perm1, *perm2;
    FILE *f;
    const char fn[] = "zzztest.tmp";
    char a[200];

    MESSAGE(0,("Testing file i/o\n"));
    pol1 = RandomPol(50);
    mat1 = RandomMat(30,29);
    perm1 = RandomPerm(50);

    f = SFOpen(fn,FM_CREATE);
    MatWrite(f,mat1);
    PermWrite(f,perm1);
    PolWrite(f,pol1);
    fclose(f);

    f = SFOpen(fn,FM_READ);
    mat2 = MatRead(f);
    perm2 = PermRead(f);
    pol2 = PolRead(f);
    if (fread(a,1,sizeof(a),f) != 0)
	ERR("missing EOF");
    fclose(f);

    if (MatCompare(mat1,mat2) != 0)
	ERR("Matrix compare failed");
    if (PolCompare(pol1,pol2) != 0)
	ERR("Polynomial compare failed");
    if (PermCompare(perm1,perm2) != 0)
	ERR("Permutation compare failed");
}





void TestPolGcd()

{
    int n;

    MtxRandomInit(0);
    MESSAGE(0,("Testing polynomial g.c.d\n"));
    for (n = 1000; n > 0; --n)
    {
	poly_t *a, *b, *gcd, *result[3];

	/* Create two random polynomials
	   ----------------------------- */
	a = RandomPol(MtxRandomInt(50));
	b = RandomPol(MtxRandomInt(50));

	/* Calculate the g.c.d.
	   -------------------- */
	if ((gcd = PolGcd(a,b)) == NULL)
	    ERR("PolGcd");
	if (PolGcdEx(a,b,result) != 0)
	    ERR("PolGcdEx");

	/* Check the result
	   ---------------- */
	if (PolCompare(gcd,result[0]) != 0)
	    ERR("PolGcd != PolGcdEx");
	PolMul(result[1],a);
	PolMul(result[2],b);
	PolAdd(result[1],result[2]);
	if (PolCompare(gcd,result[1]) != 0)
	    ERR("PolGcdEx coeffs");
	if (PolMod(a,gcd) == NULL)
	    ERR("PolMod");
	if (PolMod(b,gcd) == NULL)
	    ERR("PolMod");
	if (a->deg >= 0 || b->deg >= 0)
	    ERR("gcd");
	PolFree(a);
	PolFree(b);
	PolFree(gcd);
	PolFree(result[0]);
	PolFree(result[1]);
	PolFree(result[2]);
    }
}




void prtables()
{	int a, b;

	printf(" + ");
	for (a = 0; a < (FEL)fld; ++a)
		printf("%3d", a);
	printf("\n");
	for (a = 0; a < (int)fld; ++a)
	{       printf("%3d",a);
		for (b = 0; b < (int)fld; ++b)
		    printf("%3ld",zftoi(zadd(ftab[a],ftab[b])));
		printf("\n");
	}

	printf("\n * ");
	for (a = 0; a < (FEL)fld; ++a)
		printf("%3d", a);
	printf("\n");
	for (a = 0; a < (int)fld; ++a)
	{       printf("%3d",a);
		for (b = 0; b < (int)fld; ++b)
			printf("%3ld",zftoi(zmul(ftab[a],ftab[b])));
		printf("\n");
	}

}



static void timing()

{
    static int firsttime = 1;
    long noc = 2000;
    long count = 0;
    PTR x, y, z;
    long i, t;

    if (firsttime)
    {
	firsttime = 0;
	printf("ZZZ version: %s\n",zzzversion);
	printf("Field  Integer Extr/Ins    RowOp\n");
    }
    printf("%5ld",fld);

    zsetlen(noc);
    x = zalloc(1);
    y = zalloc(1);
    z = zalloc(1);
    MtxRandomInit(1);
    for (i = 1; i <= noc; ++i)
    {
	zinsert(x,i,ftab[MtxRandomInt(fld)]);
	zinsert(y,i,ftab[MtxRandomInt(fld)]);
    }

    for (count = 0, t = STimeUsed(); STimeUsed() - t < 5000; ++count)
    {
	register long i, k;
	for (i = k = 0; i < 10000; ++i) k += i / 17;
    }
    t = STimeUsed() - t;
    printf("%9ld",count); 
    fflush(stdout);


    /* extract/insert */
    for (count = 0, t = STimeUsed(); STimeUsed() - t < 10000; ++count)
    {
        for (i = 1; i <= noc; ++i)
	    zinsert(z,i,zextract(y,i));
    }
    printf("%9ld",count); 
    fflush(stdout);


    for (count = 0, t = STimeUsed(); STimeUsed() - t < 10000; ++count)
    {
	zaddrow(z,x);
	zaddmulrow(z,y,ftab[MtxRandomInt(fld)]);
    }
    t = STimeUsed() - t;
    printf("%9ld",count/10); 
    printf("\n");
}



static void zzztest()

{
    MESSAGE(0,("Testing with GF(%ld)\n",fld));
    MESSAGE(0,("ZZZ version: %d\n",zzzvers));
    TestReadWrite();
    TestPolGcd();
    testfield();
    testgen();
    testextractinsert();
    testfindmark();
    testaddrow();
    testrowops();
} 


int main(int argc, char *argv[])

{   int k;

    zzzvers = MtxInit();
    MtxSetCommandLine(argc, argv, &pinfo);
    while ((k = MtxGetOpt("bt")) != OPT_END)
    {   switch (k)
	{ 
	    case 'b': opt_b = 1; break;
	    case 't': opt_t = 1; break;
	}
    }
    
    for (; opt_ind < argc; ++opt_ind)
    {
	fld = atol(argv[opt_ind]);
	zsetfield(fld);
	zsetlen(TNOC);
	ftab = NALLOC(FEL,fld);
    	testitof();

    	if (opt_t)
	    prtables();
	else if (opt_b)
	    timing();
	else
	    zzztest();
	free(ftab);
    }
    return 0;
}




